#!/usr/bin/env ruby

require 'rexml/document'
require 'base64'
require 'zlib'

class Solution
	attr_accessor	:assignmentId, :coordX, :coordY, :timestamp, :valid

	def self.get_all(filename, videoId)
		xml_string = File.read(filename)
		xml_doc = REXML::Document.new(xml_string)

		solution_list = []
		xml_doc.root.each_element("/videos/video[@id='#{videoId}']/solution") do |e|
			assignmentId = e.attributes['id']
			coordX       = e.elements['coordX'].text.strip
			coordY       = e.elements['coordY'].text.strip
			timestamp    = e.elements['timestamp'].text.strip
			valid        = e.elements['valid'].text.strip == 'true'

			solution = Solution.new(assignmentId, coordX, coordY, timestamp, valid)
			solution_list << solution if solution.isValid
		end

		min_len = solution_list.map { |s| s.timestamp.length }.min
		solution_list.each { |s| s.truncateLength(min_len) }

		solution_list
	end

	def self.add(filename, videoId, solution_list)
		if File.exist?(filename)
			xml_string = File.read(filename)
			xml_doc = REXML::Document.new(xml_string)
		else
			xml_doc = REXML::Document.new
		end

		if !xml_doc.elements["/videos"]
			xml_doc.add_element('videos')
		end

		if !xml_doc.elements["/videos/video[@id='#{videoId}']"]
			e_video = xml_doc.elements['videos'].add_element('video')
			e_video.attributes['id'] = videoId
		end

		xml_doc.root.each_element("/videos/video[@id='#{videoId}']") do |e|
			solution_list.each do |solution|
				e_solution = e.add_element('solution')
				e_solution.attributes['id'] = solution.assignmentId

				e_coordX    = e_solution.add_element('coordX')
				e_coordY    = e_solution.add_element('coordY')
				e_timestamp = e_solution.add_element('timestamp')
				e_valid     = e_solution.add_element('valid')

				e_coordX.text    = solution.getCoordXString()
				e_coordY.text    = solution.getCoordYString()
				e_timestamp.text = solution.getTimestampString()
				e_valid.text     = solution.valid ? 'true' : 'false'
			end
		end

		xml_file = File.new(filename, 'w')
		xml_doc.write(xml_file, 2)
		xml_file.close
	end

	def self.remove(filename, videoId, id_list)
		if !File.exist?(filename)
			return
		end

		xml_string = File.read(filename)
		xml_doc = REXML::Document.new(xml_string)

		id_list.each do |solutionId|
			xml_doc.elements.delete_all("/videos/video[@id='#{videoId}']/solution[@id='#{solutionId}']")
		end

		xml_file = File.new(filename, 'w')
		xml_doc.write(xml_file, 2)
		xml_file.close
	end

	def self.remove_all(filename, videoId)
		if !File.exist?(filename)
			return
		end

		xml_string = File.read(filename)
		xml_doc = REXML::Document.new(xml_string)
	
		xml_doc.elements.delete_all("/videos/video[@id='#{videoId}']/solution")
	
		xml_file = File.new(filename, 'w')
		xml_doc.write(xml_file, 2)
		xml_file.close
	end

	def initialize(assignmentId, coordX, coordY, timestamp, valid)
		if !assignmentId.is_a?(String) || assignmentId == 'undefined'
			assignmentId = 'sandbox_' + (0...20).map{ ('A'..'Z').to_a[rand(26)] }.join
		end

		if coordX.is_a?(String)
			coordX = Zlib::Inflate.inflate(Base64.decode64(coordX))
			coordX = coordX.split(',').map { |v| v.to_i }
		end

		if coordY.is_a?(String)
			coordY = Zlib::Inflate.inflate(Base64.decode64(coordY))
			coordY = coordY.split(',').map { |v| v.to_i }
		end

		if timestamp.is_a?(String)
			timestamp = Zlib::Inflate.inflate(Base64.decode64(timestamp))
			timestamp = timestamp.split(',').map { |v| v.to_f }
		end

		if valid.is_a?(String)
			valid = valid == "true"
		end

		@assignmentId = assignmentId
		@coordX       = coordX
		@coordY       = coordY
		@timestamp    = timestamp
		@valid        = valid

		@h         = 0
		@cur_index = 0

		resample
	end

	def isValid
		return false if !@assignmentId.is_a?(String) || @assignmentId == ''
		return false if !@coordX.is_a?(Array) || @coordX.length < 2
		return false if !@coordY.is_a?(Array) || @coordY.length < 2
		return false if !@timestamp.is_a?(Array) || @timestamp.length < 2
		return false if @coordX.length != @coordY.length || @coordX.length != @timestamp.length
		return @valid
	end

	def positiveCoordinates
		@coordX.map! { |v| [ v, 0 ].max }
		@coordY.map! { |v| [ v, 0 ].max }
	end

	def seek(t)
		last_index = @timestamp.length - 1

		while @timestamp[@cur_index] < t && @cur_index < last_index do
			@cur_index += 1
		end

		while @timestamp[@cur_index] > t && @cur_index > 0 do
			@cur_index -= 1
		end

		if @cur_index >= last_index || @timestamp[@cur_index+1] == @timestamp[@cur_index]
			@h = 0.0
		else
			@h = (t - @timestamp[@cur_index]) / (@timestamp[@cur_index+1] - @timestamp[@cur_index])
		end
	end

	def getCoordX
		if @h == 0.0
			return @coordX[@cur_index]
		else
			return (1.0 - @h) * @coordX[@cur_index] + @h * @coordX[@cur_index+1]
		end
	end

	def getCoordY
		if @h == 0.0
			return @coordY[@cur_index]
		else
			return (1.0 - @h) * @coordY[@cur_index] + @h * @coordY[@cur_index+1]
		end
	end

	def resample(fps = 25)
		len = (@timestamp.max*fps + 1).round

		coordX    = Array.new(len)
		coordY    = Array.new(len)
		timestamp = Array.new(len)

		for i in 0..(len-1) do
			t = i.to_f/fps
			seek(t)
			coordX[i] = getCoordX
			coordY[i] = getCoordY
			timestamp[i] = t
		end
		
		@coordX    = coordX
		@coordY    = coordY
		@timestamp = timestamp
	end

	def truncateLength(new_length)
		@coordX = @coordX.slice(0, new_length)
		@coordY = @coordY.slice(0, new_length)
		@timestamp = @timestamp.slice(0, new_length)
	end

	def discardStationary(min_lag = 100)
		lag_len   = 0
		lag_start = 0

		for i in 1..(@timestamp.length-1) do
			deltaX = (@coordX[i] - @coordX[i-1]).abs
			deltaY = (@coordY[i] - @coordY[i-1]).abs

			if (deltaX <= 1 && deltaY <= 1)
				lag_len += 1
			else
				if lag_len >= min_lag
					for j in lag_start..(i-1) do
						@coordX[j] = -1
						@coordY[j] = -1
					end
				end

				lag_len   = 0
				lag_start = i
			end
		end
	end

	def getCoordXString
		coordX = @coordX.map { |i| sprintf('%d', i) }.join(',')
		Base64.encode64(Zlib::Deflate.deflate(coordX)).gsub(/[\r\n]/, '')
	end

	def getCoordYString
		coordY = @coordY.map { |i| sprintf('%d', i) }.join(',')
		Base64.encode64(Zlib::Deflate.deflate(coordY)).gsub(/[\r\n]/, '')
	end

	def getTimestampString
		timestamp = @timestamp.map { |i| sprintf('%.2f', i) }.join(',')
		Base64.encode64(Zlib::Deflate.deflate(timestamp)).gsub(/[\r\n]/, '')
	end
end

